/*
 * Mask.h
 *
 *  Created on: Jan 7, 2013
 *      Author: Brenda Javornik (e-mail: brendaj (at) nwra.com)
 */

#ifndef MASK_H_
#define MASK_H_

#include <stdbool.h>
#include "utilities.h"
#include "matrix.h"

/**
 * MASK contains a 3D boolean matrix of points to keep and
 * remove from the weights.
 * @ingroup Mask
 */
typedef struct {
  bool *data;			///< array of data values stored in row-major order
  int nX;				///< number of rows in xy-plane
  int nY;				///< number of columns in xy-plane
  int nT;				///< number of measurements
} MASK;

MASK newMask(int r, int c, int m);

void destroyMask(MASK *m);

bool isEmptyMask(const MASK *m);

MASK readMasks(char *basedir, char *filenames[], int nFiles);

void insertXYPlaneIntoMask(MATRIX xyPlane, int map, MASK *tdptr);

MATRIX extractXYPlaneFromMask(int map, MASK masks);

MATRIX extractMapZ1FromMask(int kx, int ky, const MASK *masks);

void printMask(const MASK *mask);

#endif /* MASK_H_ */
