/*
 * SuperY.h
 *
 *  Created on: Jun 16, 2011
 *      Author: Brenda Javornik (e-mail: brendaj (at) nwra.com)
 */

#ifndef SUPERY_H_
#define SUPERY_H_

	/*
	 * Format for super vector y (using z-major ordering):
	 * T[z=0, v=0]
	 * T[z=1, v=0]
	 * T[z=2, v=0]
	 * T[z=0, v=1]
	 *  ...
	 * T[z=Nz, v=2]
	 * Noise^2[a=0]
	 *  ...
	 * Noise^2[a=M]
	 */
	typedef struct {
		//dvec *y; // the real, packed super vector
		long     yLength;
		Float  *yElements;
		int Nx;
		int Ny;
		int Nz;
		int Nm;
		int Nv;
		int noiseBase;
		int RC; // nRows * nCols = Nx * Ny
	} SuperVectorY;



#endif /* SUPERY_H_ */

	void InitSuperVectorY(SuperVectorY *y, MAP_DEPTH_MATRIX m);

	void RealToDoubleComplex(Float *rData, int nelements, FloatComplex *cData);

	bool DoubleComplexToReal(FloatComplex *cData, int nelements, Float *rData);

	void unpackSuperY(const SuperVectorY *superY, MAP_DEPTH_MATRIX *AvgKern, MAP_DEPTH_MATRIX *noise,  enum DataSpace outSpace);

	bool packSuperY(const MAP_DEPTH_MATRIX *AvgKern, const MAP_DEPTH_MATRIX *noise,
			SuperVectorY *superY, enum DataSpace inSpace);
