#include <string.h>
#include <stdio.h>
#include <math.h>
#include <complex.h>
#include "matrix.h"
#include "fftw.h"
 
// f(k_x,k_y) = 1/(2*pi)^2 \sum_{x,y} e^{-i k_x x -i k_y y} g(x,y)
// g(x,y)    = (2*pi)^2 / N^2 \sum_{k_x, k_y} e^{+ i k_x x + i k_y y} f(k_x, k_y)

#define DEBUG 0

/*
 * WHERE DO WE WANT THE KX=KY=0 POINT? (0,0) = ( (N/2)+1, (N/2)+1 )
 * forward transform
 */
MATRIX mydft(const MATRIX *m, const MATRIX *mask) {

	MATRIX mk = newComplexMatrix(m->nRow, m->nCol);
	printf("inside mydft\n");

	const double ONE_OVER_TWOPI_SQUARED = 1.0/ ((2.0 * M_PI) * (2.0 * M_PI));
	double   TWOPI_OVER_L;
	if (m->nRow != m->nCol) {
		printf("Matrix must be square.\n");
	    exit(1);
		//reportError("Matrix must be square.", "mydft", FATAL);
	}
	if (mask->mtype != COMPLEX) {
		printf("Mask must be of type double complex.\n");
	    exit(1);
		//reportError("Matrix must be square.", "mydft", FATAL);
	}
	if ((mask->nCol != m->nCol) || (mask->nRow != m->nRow))  {
		printf("Mask must have the same dimensions as the matrix.\n");
	    exit(1);
		//reportError("Matrix must be square.", "mydft", FATAL);
	}

	TWOPI_OVER_L = (2.0*M_PI)/(m->nRow);  // works only for square matrices!

	int kx,ky;
	int x, y;

	double complex *kp; // pointer to output matrix data
	double complex *rp; // pointer to input matrix data
	double complex *mp;         // pointer to mask matrix data
    // xValues and yValues go from -(N/2) to (N/2) - 1
	double xValue, yValue;


	kp = mk.data;
	for (kx = 0; kx < m->nRow; kx++) {
		for (ky = 0; ky < m->nCol; ky++) {
			double vreal, vimag;
			vreal = 0.0;
			vimag = 0.0;
			rp = m->data;
			mp = mask->data;
			xValue = 0; // -(m->nRow / 2.0);
			yValue = xValue;
			printf("-----------\n");

			for (x = 0; x < m->nRow; x++) {
				for (y = 0; y < m->nCol; y++) {
//-----------
					printf("kx=%d ky=%d x=%d y=%d\n", kx, ky, x, y);
					  double xp, yp;
					  double theta;
				 	  xp = xValue * TWOPI_OVER_L;
				      yp = yValue * TWOPI_OVER_L;
				     printf("xp=%le, yp=%le\n", xp, yp);
					  theta = kx * xp + ky * yp; // these are actual values x, y not indices in matrix
printf("theta = %le\n", theta);
printf("g(%d,%d) = %le + %leI\n", x, y, creal(*rp), cimag(*rp));

					  double complex value;
					  double value1, value2;
                      value1 = cos(theta);
                      value2 = sin(theta);
                      value = value1 - value2*I;   // NOTICE: the sign here, differs from the inverse transform
                      printf("value = %le + %leI\n", creal(value), cimag(value));
                      value = *rp * value * *mp * ONE_OVER_TWOPI_SQUARED;
                      printf("value = %le + %leI\n", creal(value), cimag(value));
					  *kp += value;

					  printf("*kp=%le\n", *kp);
					  rp++;
					  mp++;
					  yValue += 1.0;
//---------------
				}
				xValue += 1.0;
			}
			kp++;
		}
	}

	print_matrix2("mk", mk, ROW_MAJOR);

	// compare to fftw
	MATRIX fromFFTW;
	fromFFTW = fftw(MatrixToFitsObject(ConvertToReal(*m)));
	print_matrix2("fromFFTW", fromFFTW, ROW_MAJOR);

	return mk;
}

/*
 * inverse transform
 */
MATRIX myidft(const MATRIX *m, const MATRIX *mask) {

	MATRIX mk = newComplexMatrix(m->nRow, m->nCol);
	printf("inside mydft\n");

	const double TWOPI_SQUARED = (2.0 * M_PI) * (2.0 * M_PI);

	double   TWOPI_OVER_L;
	if (m->nRow != m->nCol) {
		printf("Matrix must be square.\n");
	    exit(1);
		//reportError("Matrix must be square.", "mydft", FATAL);
	}
	if (mask->mtype != COMPLEX) {
		printf("Mask must be of type double complex.\n");
	    exit(1);
		//reportError("Matrix must be square.", "mydft", FATAL);
	}
	if ((mask->nCol != m->nCol) || (mask->nRow != m->nRow))  {
		printf("Mask must have the same dimensions as the matrix.\n");
	    exit(1);
		//reportError("Matrix must be square.", "mydft", FATAL);
	}

	TWOPI_OVER_L = (2.0*M_PI)/(m->nRow);  // works only for square matrices!

	int kx,ky;
	int x, y;

	double complex *kp;
	double complex *rp;
	double complex *mp;
    // xValues and yValues go from -(N/2) to (N/2) - 1
	double xValue, yValue;

    double N = m->nRow;

	rp = mk.data;
	mp = mask->data;
	xValue = 0; // -(N / 2.0);
	yValue = xValue;
	for (x = 0; x < m->nRow; x++) {
		for (y = 0; y < m->nCol; y++) {
			double vreal, vimag;
			vreal = 0.0;
			vimag = 0.0;
			kp = m->data;

			printf("-----------\n");

			double xp, yp;
		 	xp = xValue * TWOPI_OVER_L;
		    yp = yValue * TWOPI_OVER_L;


			for (kx = 0; kx < m->nRow; kx++) {
				for (ky = 0; ky < m->nCol; ky++) {
//-----------
				    printf("xp=%le, yp=%le\n", xp, yp);
					printf("x=%d y=%d kx=%d ky=%d\n", x, y, kx, ky);
					  double theta;

					  theta = kx * xp + ky * yp; // these are actual values x, y not indices in matrix
printf("theta = %le\n", theta);
printf("f(%d,%d) = %le + %leI\n", kx, ky, creal(*kp), cimag(*kp));

					  double complex value;
					  double value1, value2;
                      value1 = cos(theta);
                      value2 = sin(theta);
                      value = value1 + value2*I;  // NOTICE: the sign here, differs from the forward transform
                      printf("value = %le + %leI\n", creal(value), cimag(value));
                      value = *kp * value;
                      printf("value = %le + %leI\n", creal(value), cimag(value));
					  *rp += value;

					  printf("*rp=%le\n", *rp);
					  kp++;

//---------------
				}

			}
			printf("*mask=%le\n", *mp);
			*rp = *rp * (1.0/(N*N)) * TWOPI_SQUARED * *mp;
			printf("after applying mask *rp=%le\n", *rp);
			rp++;
			mp++;
			yValue += 1.0;
		}
		xValue += 1.0;
	}

	print_matrix2("mk", mk, ROW_MAJOR);

	// compare to fftw
	MATRIX fromFFTW;
	fromFFTW = fftwReverse(*m, 0);
	print_matrix2("fromFFTW", fromFFTW, ROW_MAJOR);

	return mk;
}


int main(int argc, char *argv[])
{
    MATRIX m, mk, mk2, mask;

    mask = newComplexMatrix(2,2);
    double complex *maskp;
    maskp = mask.data;
    maskp[0] = maskp[1] = maskp[3] = 1.0;
    maskp[2] = 0.0;

    m = newComplexMatrix(2,2);
    double complex *mp;
    mp = m.data;
    mp[0] = 3.0; mp[1] = 2.0; mp[2] = 1.0; mp[3] = 5.0;
print_matrix2("m", m, ROW_MAJOR);
    // mk = mydft(&m, &mask);

    mk2 = myidft(&m, &mask);

    print_matrix2("mk2", mk2, ROW_MAJOR);

	//MATRIX fromFFTW;
	//fromFFTW = fftw(MatrixToFitsObject(ConvertToReal(*m)));
	//print_matrix2("fromFFTW", fromFFTW, ROW_MAJOR);
}

/*
int main(int argc, char *argv[])
{
    MATRIX m, mk, mk2, mask;

    mask = newComplexMatrix(2,2);
    double complex *maskp;
    maskp = mask.data;
    maskp[0] = maskp[1] = maskp[2] = maskp[3] = 1.0;

    m = newComplexMatrix(2,2);
    double complex *mp;
    mp = m.data;
    mp[0] = 1.0; mp[1] = 1.0; mp[2] = 0; mp[3] = 1.0;
print_matrix2("m", m, ROW_MAJOR);
    mk = mydft(&m, &mask);

    mk2 = myidft(&mk, &mask);
}

*/
