/*
 * FileMapAndParameters.h
 *
 *  Created on: May 12, 2010
 *      Author: Brenda Javornik (e-mail: brendaj (at) nwra.com)
 */

#ifndef FILEMAPANDPARAMETERS_H_
#define FILEMAPANDPARAMETERS_H_

#include <stdio.h>
#include "lambda.h"
/**
 * Input parameters specific to 1d problem
 * @ingroup FileMapAndParameters
 */
typedef struct {
	char *InputKernelFileName;	///< name of kernel file
	char *InputMapFileName;		///< name of map file
} D1;

/**
 * Input parameters specific to 3d problem
 * @ingroup FileMapAndParameters
 */
typedef struct {
	char *InputKernelBaseDirName;	///< base directory of kernel files
	char *InputMapBaseDirName;		///< base directory of map files
	char *InputMaskBaseDirName;		///< base directory of mask files
	char *AvgKernelIndices;			///< Avg Kernel specification line
	int nKernelMapFiles;			///< number of kernel-map pairs
	char **KernelFilenames;			///< list of kernel file names
	char **MapFilenames;			///< list of map file names
	char **MaskFilenames;			///< list of mask file names
} D3;

/**
 * Information about the target functions used
 * in OLA base inversions.
 * @ingroup FileMapAndParameters
 */
typedef struct {
	int nTargetFiles;				///< number of target depths to solve for
	char **TargetFilenames;			///< list of target depth file names
} TargetInfo;
/**
 * Type of calculation: can be inversion, forward or calculate q.
 * @ingroup FileMapAndParameters
 */
typedef enum {RLS_INV, OLA_INV, LSQR_INV, FWD_CALC, CALCQ_CALC} CalculationType;

/**
 * Input parameters for both 1d and 3d problem.
 * This structure is filled while parsing the input file
 * and returned to the calling function.
 * @ingroup FileMapAndParameters
 */
typedef struct {

	D1 d1;		///< 1d specific parameters
	D3 d3;		///< 3d specific parameters


	CalculationType whichCalc; ///< Which type of calculation to perform
	// modifiers to the calculation.  Constrained modifies LSQR.
	// Crosstalk modifies OLA and LSQR
	bool constrained;	///< Constrained LSQR inversion
	bool spread;		///< spread of the weights in space for OLA based inversions
	bool xt;			///< crosstalk term
	bool noMaps;		///< exclude maps from OLA based inversions

	// used for 3D problem
	char *outputBaseDirName;		///< base directory for all output files
	char *outputBaseDirNameOrig;	///< original base directory for all output files
	char *InputICovarFileName;		///< Inverse covariance file name
	char *InputRegFileName;			///< Regularization file name
	LambdaSet lambdaSet1;			///< Range of values for first lambda parameter
	LambdaSet lambdaSet2;			///< Range of values for second lambda parameter
	LambdaSet lambdaSet3;			///< Range of values for third lambda parameter
	LambdaSet lambdaSet4;			///< Range of values for fourth lambda parameter
	char *OptionalFile; 			///< Optional file
	TargetInfo targetInfo;			///< Files containing target function depths
	char *InitialGuessFile;			///< Initial guess for LSQR/OLA algorithm
	int scatterer;					///< Scatterer of interest for 3dv OLA
} InputParms;

int getInputFromFile(char *inputFilename, InputParms *iParms, bool copyInputFiles);

void freeInputParms(InputParms ip);

#endif /* FILEMAPANDPARAMETERS_H_ */
