/*
 * InputData.c
 *
 *  Created on: Feb 7, 2012
 *      Author: Brenda Javornik (e-mail: brendaj (at) nwra.com)
 */

#include "InputData.h"

/**
 * Make new instance of InputData and initialize it.
 * @return Pointer to InputData.
 */
InputData *newInputData() {
	InputData *id;
	id = (InputData *) td_malloc(sizeof(InputData));
	id->C.data = NULL;
	id->C1d.data = NULL;
	id->K.data = NULL;
	id->K1d.data = NULL;
	id->M.data = NULL;
	id->M1d.data = NULL;
	id->R.data = NULL;
	id->Theta.data = NULL;
	id->mask.data = NULL;
	id->IG.data = NULL;
	return id;
}

/**
 * Free memory associated with InputData structure.
 * @param[in] inData A pointer to InputData structure to free.
 */
void destroyInputData(InputData *inData) {

	if (!isEmptyMask(&(inData->mask))) {
		destroyMask(&(inData->mask));
	}

	if (!isEmptyMapDepthMatrix(&(inData->IG)))
		destroyMapDepthMatrix(inData->IG);

	if (!isEmptyMapDepthMatrix(&(inData->C))) {
		destroyMapDepthMatrix(inData->C);
	}

	if (!isEmptyMapDepthMatrix(&(inData->K)))
		destroyMapDepthMatrix(inData->K);

	if (!isEmptyMapDepthMatrix(&(inData->M)))
		destroyMapDepthMatrix(inData->M);

	if (inData->Theta.data != NULL)
		freeMatrix(inData->Theta);

}

