%{
#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>

#include "myptypes.h"
#include "utilities.h"
#include "errorHandler.h"
#include "FileMapAndParameters.h"
#include "List.h"
#include "lambda.h"
#include "ProblemDimension.h"
#include "parserErrorMessages.h"
	
#define MYDEBUG 0
#define MAX_AVG_KERN_LINE 200
#define YYSTYPE char *
 
YYLTYPE yylloc; 
const char *expecting = MissingProblemDefn;
char moduleName[] = "parser";

bool definitely = false;

InputParms *inputParms;

bool copyInput = false;

List kList = NULL;
List mList = NULL;
List mkList = NULL;
List tList = NULL;

int ParseError = 0;

int depthCount = 0;
bool seenMinus1 = false;

char AvgKernLine[MAX_AVG_KERN_LINE];	

int iflog = 0;
int seed = 0;


int yywrap()
{
        return 1;
} 
 
void ProcessOutputBaseDirLine(char *line);
void PullKernelFileNames();
void ConvertFileNamesToList(bool readMask,  bool readMap, int nkernels);
void yyerror(const char *str);

%}

%token T_AVG T_KERN T_1D T_2D T_3D T_3D_V T_3D_VX T_3D_VY T_3D_VZ T_LSQR T_CLSQR T_OLA T_RLS T_FORWARD T_CALCQ T_INTEGER T_REAL T_FILENAME T_LOG T_SEED EOL

%%

input_file: 
        rls_1d |
         rls_3d | 
         rls_3dv | 
         ola_2d | 
         ola_3d | 
         ola_3dv | 
         lsqr_2d |
         lsqr_3d |
         lsqr_3dv |
         forward_3d |
         forward_3dv |
         calcq;

        
rls_1d:
	rls_1d_spec out_base_dir map_file covar_file kern_file theta_file parm ending
	{
		inputParms->d1.InputMapFileName = td_malloc(strlen($3)+1);
		inputParms->d1.InputKernelFileName = td_malloc(strlen($5)+1);
		strncpy(inputParms->d1.InputMapFileName, $3, strlen($3)+1);
		strncpy(inputParms->d1.InputKernelFileName, $5, strlen($5)+1);
		
		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $7;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
	};  
	
	
rls_3d:
		rls_3d_spec everybody scalar_3d_body theta_file avg_kern loop_parms ending
		{};      
		
rls_3dv:
		rls_3dv_spec everybody vector_3d_body theta_file avg_kern loop_parms ending
		{};  
        
ola_2d:
		ola_2d_spec everybody scalar_3d_body loop_parms ending
		{};
		
ola_3d:
		ola_3d_spec everybody scalar_3d_body theta_file loop_parms EOL target_file_list ending
		{};
		
ola_3dv:
		ola_3dv_spec everybody vector_3d_body theta_file loop_parms3 EOL target_file_list ending
		{};
		
lsqr_2d:
		lsqr_2d_spec everybody mask_base_dir lsqr_2d_km_pairs covar_file initial_guess_file loop_parms ending
		{
		};
		
lsqr_3d:
		lsqr_3d_spec everybody mask_base_dir lsqr_2d_km_pairs covar_file theta_file initial_guess_file loop_parms EOL target_file ending
		{
		};
		
lsqr_3dv:
		lsqr_3dv_spec everybody mask_base_dir lsqr_3dv_km_pairs covar_file theta_file initial_guess_file loop_parms3 EOL target_file ending
		{
		};

forward_3d:
	//  NUMBER OF EXPECTED KERNELS GOES WITH DIMENSION PLUS, EACH LINE OF KERNEL FILE IS A MAP INDEX
	//  Noise amplitude files take the place of map files. 
	//  There is one amplitude file for each kernel. 
	//  Theta file has the same meaning as in inversions.
	//  covar_file is really the model data file.
	//  Looping parameters scale the noise amplitude; a zero loop parameter == no noise.
	forward_3d_spec everybody scalar_3d_body theta_file seed parm ending
	{
		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $6;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
		
		// record the seed
		if (seed) {
			lsPtr = (LambdaSet *) $5;
			inputParms->lambdaSet2 = *lsPtr;
			freeLambdaSet(lsPtr);
		} else {
			inputParms->lambdaSet2.min = 0;
			inputParms->lambdaSet2.max = 0;
			inputParms->lambdaSet2.n = 0;
			inputParms->lambdaSet2.log = 0;
		}
	};
	
forward_3dv:
	//  NUMBER OF EXPECTED KERNELS GOES WITH DIMENSION PLUS, EACH LINE OF KERNEL FILE IS A MAP INDEX
	// covar_file is really the model data file
	// forward_3dv_spec out_base_dir kern_base_dir kernels3 covar_file theta_file parm ending
	forward_3dv_spec everybody vector_3d_body theta_file seed parm ending
	{
		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $6;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
		
		// record the seed
		if (seed) {
			lsPtr = (LambdaSet *) $5;
			inputParms->lambdaSet2 = *lsPtr;
			freeLambdaSet(lsPtr);
		} else {
			inputParms->lambdaSet2.min = 0;
			inputParms->lambdaSet2.max = 0;
			inputParms->lambdaSet2.n = 0;
			inputParms->lambdaSet2.log = 0;
		}
	};
	
calcq:
	//  kernel_base_dir is really the weight base directory
	//  map_base_dir has the same meaning
	//  just get list of weight files and list of map files
	calcq_spec 	out_base_dir kern_base_dir weight_file_name map_base_dir map_files convert_files ending
	{

	};


rls_1d_spec:
	T_1D rls_module EOL
	{
		setProblemDimension(PDC.D1);
		inputParms->whichCalc = RLS_INV;
		expecting = MissingBaseDir;
	};
		
ola_2d_spec:
		T_2D ola_module EOL
		{
			setProblemDimension(PDC.D2);
			inputParms->constrained = false;
			inputParms->whichCalc = OLA_INV;
			expecting = MissingBaseDir;
		};
		
		
ola_3dv_spec:
		ola_3dv_scat ola_module EOL
		{
			setProblemDimension(PDC.D5);
			inputParms->constrained = false;
			inputParms->whichCalc = OLA_INV;
			expecting = MissingBaseDir;
		};
	
rls_3d_spec: 
		T_3D rls_module EOL
		{
			setProblemDimension(PDC.D3);
			if (definitely) {
				inputParms->constrained = false;
				inputParms->whichCalc = RLS_INV;
			}
			expecting = MissingBaseDir;
		};
		
ola_3d_spec: 
		T_3D ola_module EOL
		{
			setProblemDimension(PDC.D3);
			if (definitely) {
				inputParms->constrained = false;	
				inputParms->whichCalc = OLA_INV;
			}
			expecting = MissingBaseDir;
		};
		
rls_3dv_spec:
		T_3D_V rls_module EOL
		{
			setProblemDimension(PDC.D5);
			inputParms->constrained = false;	
			inputParms->whichCalc = RLS_INV;
			expecting = MissingBaseDir;
		};
	 
lsqr_2d_spec:
		T_2D which_lsqr EOL
		{
			setProblemDimension(PDC.D2);	
			inputParms->whichCalc = LSQR_INV;
			inputParms->InitialGuessFile = NULL;
			expecting = MissingBaseDir;
			
		};
		
lsqr_3d_spec:
		T_3D which_lsqr EOL
		{
			setProblemDimension(PDC.D3);	
			inputParms->whichCalc = LSQR_INV;
			inputParms->InitialGuessFile = NULL;
			expecting = MissingBaseDir;
		};
		
lsqr_3dv_spec:
		ola_3dv_scat which_lsqr EOL
		{
			if (MYDEBUG) printf("inside lsqr_3dv_spec\n");
			setProblemDimension(PDC.D5);	
			inputParms->whichCalc = LSQR_INV;
			inputParms->InitialGuessFile = NULL;		
			expecting = MissingBaseDir;
		};
		
which_lsqr:
        plain_lsqr | constrained_lsqr
		
plain_lsqr:
        T_LSQR
        {
        	inputParms->constrained = false;
        };
        
constrained_lsqr:
		 T_CLSQR
		 {
			inputParms->constrained = true;
		 };
		
forward_3d_spec:
		T_3D T_FORWARD EOL
		{
			if (MYDEBUG) printf("inside forward_3d_spec\n");
			setProblemDimension(PDC.D3);
			inputParms->constrained = false;
			inputParms->whichCalc = FWD_CALC;
			expecting = MissingBaseDir;
			inputParms->OptionalFile = NULL;
			if (MYDEBUG) printf("leaving forward_3d_spec\n");
		};
		 
forward_3dv_spec:
		T_3D_V T_FORWARD EOL
		{
			if (MYDEBUG) printf("inside forward_3dv_spec\n");
			setProblemDimension(PDC.D5);
			inputParms->constrained = false;
			inputParms->whichCalc = FWD_CALC;
			expecting = MissingBaseDir;
			inputParms->OptionalFile = NULL;
			if (MYDEBUG) printf("leaving forward_3dv_spec\n");
		};
		
calcq_spec:
		T_CALCQ EOL
		{
			if (MYDEBUG) printf("inside calcq_spec\n");
			setProblemDimension(PDC.D3);
			inputParms->constrained = false;
			inputParms->whichCalc = CALCQ_CALC;
			expecting = MissingBaseDir;
			inputParms->OptionalFile = NULL;
			if (MYDEBUG) printf("leaving calcq_spec\n");
		};
		
		
scalar_3d_body:
		km_pairs covar_file;
		
vector_3d_body:
		k3m_pairs covar_file;
		
everybody:
	out_base_dir kern_base_dir map_base_dir;
		
ola_3dv_scat:
	ola_3dv_x | ola_3dv_y | ola_3dv_z;
	
ola_3dv_x:
	T_3D_VX
	{
		inputParms->scatterer = 0;
	};
	
ola_3dv_y:
	T_3D_VY
	{
		inputParms->scatterer = 1;
	};
	
ola_3dv_z:
	T_3D_VZ
	{
		inputParms->scatterer = 2;
	};
		 
ola_module:
		T_OLA { definitely = true;};
		
rls_module:
		T_RLS { definitely = true;};
		
avg_kern:
		avg_kern_keywords depths EOL
		{
			inputParms->d3.AvgKernelIndices = AvgKernLine;
			if (inputParms->whichCalc == RLS_INV) expecting = MissingLambdaRange;
			else expecting = MissingMuSigmaRange;
		};
		
avg_kern_keywords:
	T_AVG T_KERN
	{
		strcat(AvgKernLine, "Avg ");
		strcat(AvgKernLine, "Kernel");
		expecting = "Expecting positive integers for averaging kernel depths or -1 for no averaging kernels";
	};


	
depths: 
	| depths depth;
	
depth:
	T_INTEGER 
	{
		// depthCount keeps the number of depths seen so far.
		// Only the first depth can be -1 and if it is, then no other numbers can follow
		// If the first depth is not -1, then all the other numbers must be positive
		// and in any order.
		if (MYDEBUG) printf("count %1d: %s\n", depthCount, $1);
		int value;
		if (sscanf($1, "%d", &value) != 1) {
			reportErrorStr(ExpectedPositiveInteger, $1, moduleName, WARNING);
			ParseError = 1;
		} else {
			// A number was read just fine. Let's see what it is
			if (depthCount == 0) {
				// allow only -1 or positive values > 0
				if (value == -1)
					seenMinus1 = true;
				if ((value < -1) || (value == 0)) {
					reportErrorInt(ExpectedPositiveInteger, value, moduleName, WARNING);
					ParseError = 1;
				}
			} else {
				// count > 0, allow only values > 0 unless we've seen -1, then allow no values
				if (value < 1) {
					reportErrorInt(ExpectedPositiveInteger, value, moduleName, WARNING);
					ParseError = 1;
				}
				if (seenMinus1) {
					reportErrorInt(ExtraCharsAfterMinus1, value, moduleName, WARNING);
					ParseError = 1;
				}
			} 
		}

		depthCount += 1;
		strcat(AvgKernLine, " ");
		if (strlen($1) + strlen(AvgKernLine) + 1 > MAX_AVG_KERN_LINE) {
		  reportErrorInt(AvgKernSpecTooLong, MAX_AVG_KERN_LINE, moduleName, FATAL);
		}
		strcat(AvgKernLine, $1);
	};
        



		 
lsqr_2d_km_pairs:
	lsqr_2d_km_pair | lsqr_2d_km_pairs lsqr_2d_km_pair 
	{
		expecting = MissingCovarFile;
	};
	
lsqr_2d_km_pair:
	kern_file_name map_file_name mask_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
			mList = newList();
			mkList = newList();
		} 
		char *one;
		int len;
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);		

		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, mList);
		
		len = strlen($3)+1;
		one = td_malloc(len);
		strncpy(one, $3, len);
		insertList((char *) one, mkList);		

	};
	
lsqr_3dv_km_pairs:
	lsqr_3dv_km_pair | lsqr_3dv_km_pairs lsqr_3dv_km_pair 
	{
		expecting = MissingCovarFile;
	};
	
lsqr_3dv_km_pair:
	kern_file_name kern_file_name kern_file_name map_file_name mask_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
			mList = newList();
			mkList = newList();
		} 

		int len;
		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);
		
		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, kList);
		
		len = strlen($3)+1;
		one = td_malloc(len);
		strncpy(one, $3, len);
		insertList((char *) one, kList);
		
		len = strlen($4)+1;
		one = td_malloc(len);
		strncpy(one, $4, len);
		insertList((char *) one, mList);
		
		len = strlen($5)+1;
		one = td_malloc(len);
		strncpy(one, $5, len);
		insertList((char *) one, mkList);
		
	};
	
km_pairs:
	| km_pairs km_pair
	{
	};
	
km_pair:
	kern_file_name map_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
			mList = newList();
		} 

		int len;
		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);
		
		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, mList);
		
	};
	
k3m_pairs:
	| k3m_pairs k3m_pair
	{
	};
	
k3m_pair:
	kern_file_name kern_file_name kern_file_name map_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
			mList = newList();
		} 

		int len;
		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);
		
		len = strlen($2)+1;
		one = td_malloc(len);
		strncpy(one, $2, len);
		insertList((char *) one, kList);
		
		len = strlen($3)+1;
		one = td_malloc(len);
		strncpy(one, $3, len);
		insertList((char *) one, kList);
		
		len = strlen($4)+1;
		one = td_malloc(len);
		strncpy(one, $4, len);
		insertList((char *) one, mList);
		
	};

// For calcq, weight and map file names; each can be a list; 
// ah-ha!  I have a kernel list (kList) and a map list (mList).	
// for calcQ, keep the weight files in the kList.
weight_file_name:
	kern_file_name EOL
	{
		if (kList == NULL) {
			kList = newList();
		} 

		int len;
		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, kList);
	};

map_files:
	| map_files calcq_map_file_name
	{
	};
			
// for calcQ, keep the map files in the mList
calcq_map_file_name: 
	map_file_name EOL
	{
	    if (mList == NULL) {
			mList = newList();
		} 

		int len;
		char *one; 
		len = strlen($1)+1;
		one = td_malloc(len);
		strncpy(one, $1, len);
		insertList((char *) one, mList);
		
	};
	
convert_files:
	{
		bool mask = false;
		bool map = true;
		int nKernels = 1;
		ConvertFileNamesToList(mask, map, nKernels);
	}

loop_parms:
	parm parm
	{

		LambdaSet *lsPtr;
		lsPtr = (LambdaSet *) $1;
		inputParms->lambdaSet1 = *lsPtr;
		freeLambdaSet(lsPtr);
		
		lsPtr = (LambdaSet *) $2;
		inputParms->lambdaSet2 = *lsPtr;
		freeLambdaSet(lsPtr);		

    	inputParms->xt = false;  
		
		if ((getProblemDimension() >= PDC.D3) && ((inputParms->whichCalc == OLA_INV) || (inputParms->whichCalc == LSQR_INV)))
			expecting = Missing1dTargetFunc;

	};
	
loop_parms3:
	loop_parms others
//	loop_parms xt
	{		
		if ((getProblemDimension() >= PDC.D3) && ((inputParms->whichCalc == OLA_INV) || (inputParms->whichCalc == LSQR_INV)))
			expecting = Missing1dTargetFunc;
	};
	
others:
	| 
	{ 
    	inputParms->xt = false;  
    	if (MYDEBUG) printf("setting xt false\n"); 
    	inputParms->spread = false;  
    	if (MYDEBUG) printf("setting spread of weights to zero\n"); 
    } 
	xte
	
xte:
	xt epsilon
	
epsilon:
    |	
    { 
    	inputParms->spread = false;  
    	if (MYDEBUG) printf("setting spread of weights to zero\n"); 
    } 
    parm
    {
        // if this is a valid parameter speck, then this is the signal to 
        // use epsilon (spread of the weights in space) in the inversion calculation
        LambdaSet *lsPtr;
        lsPtr = (LambdaSet *) $2;
		inputParms->lambdaSet4 = *lsPtr;
		freeLambdaSet(lsPtr);	
        inputParms->spread = true;
        if (MYDEBUG) printf("setting spread to (zero or true)\n");
    };
	
xt:
    parm
    {
        // if this is a valid parameter speck, then this is the signal to 
        // include crosstalk in the inversion calculation
        LambdaSet *lsPtr;
        lsPtr = (LambdaSet *) $1;
		inputParms->lambdaSet3 = *lsPtr;
		freeLambdaSet(lsPtr);	
        inputParms->xt = true;
        if (MYDEBUG) printf("setting xt true\n");
    };

/* previous version of xt    
xt:
    |	
    { 
    	inputParms->xt = false;  
    	if (MYDEBUG) printf("setting xt false\n"); 
    } 
    parm
    {
        // if this is a valid parameter speck, then this is the signal to 
        // include crosstalk in the inversion calculation
        LambdaSet *lsPtr;
        lsPtr = (LambdaSet *) $2;
		inputParms->lambdaSet3 = *lsPtr;
		freeLambdaSet(lsPtr);	
        inputParms->xt = true;
        if (MYDEBUG) printf("setting xt true\n");
    };
*/

ending:
	| ending an_eol
	{

	};
	
an_eol:
	EOL
	
parm:
	number number step
	{
		
		LambdaSet *lsPtr;
		double min, max;
		sscanf($1, "%lf", &min);
		sscanf($2, "%lf", &max);
		
		// make some semantic checks on the min and max
		if (MYDEBUG) printf("checking range %lf %lf\n", min, max);
		if (max < min) {
			ParseError = 1;
			reportErrorInt(RangeMinGreaterMax, min, moduleName, ERROR);
		}

		if (iflog == 0) {
			int n;
		    sscanf($3, "%d", &n);
			if (n <= 0) {
				ParseError = 1;
				reportErrorInt(RangeNLessEqZero, n, moduleName, ERROR); 
			}
		    lsPtr = newLambdaSet(min, max, n, iflog);
		} else {
		    // the number of steps doesn't matter for log; newLambdaSet will 
		    // determine the number of steps
			lsPtr = newLambdaSet(min, max, 1, iflog);
			// reset the iflog flag
			iflog = 0;
		}
		    
		// return a pointer to this lambda set
		$$ = (char *) lsPtr;
	};
	
step:
	 logstep | T_INTEGER;
	
logstep: 
	 T_LOG
	 {
		 iflog = 1;
	 };
	 
seed:
	| seed_def;
	
seed_def:
	T_SEED T_INTEGER EOL 
	{
	    // stuff the seed into the min and max value of a lambda set

		LambdaSet *lsPtr;
		double min;
		if (sscanf($2, "%lf", &min) != 1) {
		    expecting = UnsignedIntForSeed;
		    yyerror(expecting);
		}

        // check that seed is an unsigned integer
        if ((min < 0) || ((min - (int) min) > 0)) {
		    expecting = UnsignedIntForSeed;
		    yyerror(expecting);
		}
		unsigned int seedVal;
		seedVal = (unsigned int) min;
        reportErrorInt(FoundSeedMsg, seedVal, moduleName, INFO);
		lsPtr = newLambdaSet(min, min, 1, 0);
		    
		// return a pointer to this lambda set
		$$ = (char *) lsPtr;
		seed = 1;
	};

number:
	T_REAL | T_INTEGER;

		
out_base_dir:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("Outbasedir: %s\n", $1);
		// keep the original output directory name		
		inputParms->outputBaseDirNameOrig = td_malloc(strlen(yylval)+1);
		strncpy(inputParms->outputBaseDirNameOrig, yylval, strlen(yylval)+1);
		
		char *nextEmptyDir;
		nextEmptyDir = getNextEmptyDirectory(inputParms->outputBaseDirNameOrig);
		// free original directory name
		// td_free(inputParms->outputBaseDirName, strlen(yylval)+1);

		
		// allocate space for final directory name
		//inputParms->outputBaseDirName = td_malloc(strlen(nextEmptyDir)+1);
	    inputParms->outputBaseDirName = nextEmptyDir;
	    if (MYDEBUG) printf("final outbasedir:%s|\n", inputParms->outputBaseDirName);
		
		if (getProblemDimension() >= PDC.D2)
			expecting = MissingKernBaseDir;
		else
			expecting = MissingMapFile;
	};
	
mask_base_dir:
	T_FILENAME EOL
	{
		int len;
		len = strlen(yylval)+1;
		inputParms->d3.InputMaskBaseDirName = td_malloc(len);
		strncpy(inputParms->d3.InputMaskBaseDirName, yylval, len);
		if (!isReadable(inputParms->d3.InputMaskBaseDirName))
			ParseError = 1;			
		expecting = MissingKernMapPairs;
	};
	
kern_base_dir:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("Kerbasedir: %s\n", $1);
		int len;
		len = strlen(yylval)+1;
		inputParms->d3.InputKernelBaseDirName = td_malloc(len);
		strncpy(inputParms->d3.InputKernelBaseDirName, yylval, len);
   		//inputParms->d3.InputKernelBaseDirName = (char *) yylval;
		if (!isReadable(inputParms->d3.InputKernelBaseDirName))
			ParseError = 1;
		expecting = MissingMapBaseDir;
	};
	
map_base_dir:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("Mapbasedir: %s\n", $1);
		int len;
		len = strlen(yylval)+1;
		inputParms->d3.InputMapBaseDirName = td_malloc(len);
		strncpy(inputParms->d3.InputMapBaseDirName, yylval, len);
		//inputParms->d3.InputMapBaseDirName = (char *) yylval;
		if (!isReadable(inputParms->d3.InputMapBaseDirName))
			ParseError = 1;
		if (inputParms->whichCalc == LSQR_INV)
			expecting = MissingMaskBaseDir;
		else
			expecting = MissingKernMapPairs;
	};
	
mask_file_name:
	T_FILENAME
	{
		// verify file exists and is readable
		char *fullFileName = constructFileName(inputParms->d3.InputMaskBaseDirName, $1); 
		if (!isReadable(fullFileName))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles(fullFileName, inputParms->outputBaseDirName);
		td_free(fullFileName, strlen(fullFileName) + 1);
		//printf("MskFileName: %s\n", $1);
	};
	
// 	>>>>>>>>> WHY ARE THERE TWO VERSIONS OF KERNEL AND MAP FILE NAMES?
// Because kern_file and map_file are for RLS 1d and there are no base directories for these files.
kern_file_name:
	T_FILENAME
	{
		// verify file exists and is readable
		char *fullFileName = constructFileName(inputParms->d3.InputKernelBaseDirName, $1); 
		if (!isReadable(fullFileName))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles(fullFileName, inputParms->outputBaseDirName);
		td_free(fullFileName, strlen(fullFileName) + 1);
		if (MYDEBUG) printf("KerFileName: %s\n", $1);
	};

kern_file:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("KerFileName: %s\n", $1);
		// verify kernel file exists and is readable
		if (!isReadable($1))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles($1, inputParms->outputBaseDirName);
		expecting = MissingRegFile;
	};
	
map_file_name:
	T_FILENAME
	{
		// verify file exists and is readable
		char *fullFileName = constructFileName(inputParms->d3.InputMapBaseDirName, $1); 
		if (!isReadable(fullFileName))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles(fullFileName, inputParms->outputBaseDirName);
		td_free(fullFileName, strlen(fullFileName) + 1);
		if (MYDEBUG) printf("MapFileName: %s\n", $1);
		if (inputParms->whichCalc == LSQR_INV)
			expecting = MissingMaskFile;
	};

	
map_file:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("MapFileName: %s\n", $1);
		// verify map file exists and is readable
		if (!isReadable($1))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles($1, inputParms->outputBaseDirName);
		expecting = MissingCovarFile;
	};
	
theta_file:
	T_FILENAME EOL
	{
		// inputParms->InputRegFileName = (char *) yylval;
		inputParms->InputRegFileName = td_malloc(strlen(yylval)+1); // (char *) yylval;
		strncpy(inputParms->InputRegFileName, yylval, strlen(yylval)+1); 
		// verify theta file exists and is readable
		if (!isReadable(inputParms->InputRegFileName))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles(inputParms->InputRegFileName, inputParms->outputBaseDirName);
		int dim;
		dim = getProblemDimension();
		if (dim == PDC.D1)
			expecting = MissingLambdaRange;
		else {
			if (inputParms->whichCalc == RLS_INV)
				expecting = MissingAvgKern;
			else if (inputParms->whichCalc == FWD_CALC)
			    expecting = MissingSeedOrNoiseScale;
			else
				expecting = MissingMuSigmaRange;
		}
	};

target_file_list:
	target_file | target_file_list ending target_file

target_file:
	T_FILENAME
	{
		int len;
		//len = strlen(yylval)+1;
		//inputParms->OptionalFile = td_malloc(len);
		//strncpy(inputParms->OptionalFile, yylval, len);
				
		//inputParms->OptionalFile = (char *) yylval;
		printf("target file: %s\n", $1);
		// verify target file exists and is readable
		if (!isReadable($1)) // inputParms->OptionalFile))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles($1, inputParms->outputBaseDirName);
			//copyFiles(inputParms->OptionalFile, inputParms->outputBaseDirName);
			
	//----------
		if (tList == NULL) {
			tList = newList();
		} 

		char *one;
		
		len = strlen($1)+1;
		one = td_malloc(len);
		printf("target_file length = %d\n", len);
		strncpy(one, $1, len);
		insertList((char *) one, tList);
		
	//--------------
	// */
	};

/* original
target_file:
	T_FILENAME
	{
		int len;
		len = strlen(yylval)+1;
		inputParms->OptionalFile = td_malloc(len);
		strncpy(inputParms->OptionalFile, yylval, len);
		//inputParms->OptionalFile = (char *) yylval;
		//printf("target file: %s\n", $1);
		// verify target file exists and is readable
		if (!isReadable(inputParms->OptionalFile))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles(inputParms->OptionalFile, inputParms->outputBaseDirName);
	};
end original */
	
initial_guess_file:
	| T_FILENAME EOL
	{
		int len;
		len = strlen(yylval)+1;
		inputParms->InitialGuessFile = td_malloc(len);
		strncpy(inputParms->InitialGuessFile, yylval, len);
		//inputParms->InitialGuessFile = (char *) yylval;
		reportErrorStr(FoundInitialGuessMsg, $1, moduleName, INFO);
		// verify initial guess file exists and is readable
		if (!isReadable(inputParms->InitialGuessFile))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles(inputParms->InitialGuessFile, inputParms->outputBaseDirName);
	};
		
covar_file:
	T_FILENAME EOL
	{
		if (MYDEBUG) printf("CovFile: %s\n", $1);
		inputParms->InputICovarFileName = td_malloc(strlen($1)+1);
		strncpy(inputParms->InputICovarFileName, $1, strlen($1)+1);
		
		// verify covariance file exists and is readable
		if (!isReadable(inputParms->InputICovarFileName))
			ParseError = 1;
		if (!ParseError && copyInput)
			copyFiles(inputParms->InputICovarFileName, inputParms->outputBaseDirName);
		
		int dim;
		dim = getProblemDimension();
		if (dim != PDC.D1) {
			bool mask = false;
			bool map = true;
				if (inputParms->whichCalc == LSQR_INV)// (inputParms->lsqr) 
					mask = true;
				int nKernels = 1;
				if (dim == PDC.D5)
					nKernels = 3;
				ConvertFileNamesToList(mask, map, nKernels);
		}

		if (dim == PDC.D1)
			expecting = MissingKernFile;
		if (dim >= PDC.D3)
			expecting = MissingThetaFile;
	};
%%

void ConvertFileNamesToList(bool readMask, bool readMap, int nkernels) {
	if (MYDEBUG) printf("inside ConvertFileNamesToList\n");
			
		// convert filenames from List to char **
		int nkmFiles = getLengthList(mList);
		if (nkmFiles <= 0) {
			yyerror("No kernel-map pairs found");
		} else {
			inputParms->d3.nKernelMapFiles = nkmFiles;
			inputParms->d3.KernelFilenames = (char **) td_malloc(nkmFiles * nkernels * sizeof(char*));
			inputParms->d3.MapFilenames = (char **) td_malloc(nkmFiles * sizeof(char*));
			ListIterator kli = newListIterator(kList);
			ListIterator mli = newListIterator(mList);
	
			int i;
			for (i=nkmFiles-1; i>=0; i--) {
				inputParms->d3.MapFilenames[i] = listIteratorNext(mli);
			}
			for (i=(nkmFiles*nkernels)-1; i>=0; i--) {
				inputParms->d3.KernelFilenames[i] = listIteratorNext(kli);
			}
			freeList(kList);
			freeList(mList);
	
			if (readMask) {
				inputParms->d3.MaskFilenames = (char **) td_malloc(nkmFiles * sizeof(char*));
				ListIterator mli = newListIterator(mkList);
				for (i=nkmFiles-1; i>=0; i--) {
					inputParms->d3.MaskFilenames[i] = listIteratorNext(mli);
				}
				freeList(mkList);
			}
		}
		if (MYDEBUG) printf("leaving ConvertFileNamesToList\n");
}

// one function for each kernels, maps and masks 
void PullKernelFileNames() {
	if (MYDEBUG) printf("inside PullKernelFileNames\n");
	
		int dim;
		dim = getProblemDimension();
		int nKernels = 1;
		if (dim == PDC.D5)
			nKernels = 3;
			
		// convert filenames from List to char **
		int nkmFiles = getLengthList(kList)/nKernels;
		if (nkmFiles <= 0) {
			yyerror("No kernel-map pairs found");
		} else {
			inputParms->d3.nKernelMapFiles = nkmFiles;
			inputParms->d3.KernelFilenames = (char **) td_malloc(nkmFiles * nKernels * sizeof(char*));
			ListIterator kli = newListIterator(kList);
	
			int i;
			for (i=(nkmFiles*nKernels)-1; i>=0; i--) {
				inputParms->d3.KernelFilenames[i] = listIteratorNext(kli);
			}
			freeList(kList);
		}
		if (MYDEBUG) printf("leaving PullKernelFileNames\n");
}
        
void ConvertTargetFileNamesToList() {  // bool readMask, bool readMap, int nkernels) {
	    if (MYDEBUG) printf("inside ConvertTargetFileNamesToList\n");

		// convert filenames from List to char **
		if (tList == NULL) return;
		int ntFiles = getLengthList(tList);
		if (ntFiles <= 0) {
			yyerror("No target file found");
		} else {
			inputParms->targetInfo.nTargetFiles = ntFiles;
			inputParms->targetInfo.TargetFilenames = (char **) td_malloc(ntFiles * sizeof(char*));
			ListIterator mli = newListIterator(tList);
	
			int i;
			for (i=ntFiles-1; i>=0; i--) {
				inputParms->targetInfo.TargetFilenames[i] = listIteratorNext(mli);
			}
			freeList(tList);
		}
		if (MYDEBUG) printf("leaving ConvertTargetFileNamesToList\n");
}

void yyerror(const char *str)
{
        fprintf(stderr,"Error: %s near line %d\n",expecting, yylloc.first_line+1);
        ParseError = 1;
}

extern FILE *yyin; 


// YYLTYPE yylloc = {1,1,1,1};


//main()
// return 1 for success; return 0 if error
int getInputFromFile(char *inputFilename, InputParms *iParms, bool copyInputFiles)
{

  FILE *fp=fopen(inputFilename,"r");
  if (fp > 0) {
	  if (MYDEBUG) reportError(InputFileOpenedMsg, moduleName, INFO);
  }
  else {
	  reportError(InputFileOpenFailedMsg, moduleName, FATAL);
	  ParseError = 1;
	  return ParseError;
  }
        yyin=fp;
  
  		AvgKernLine[0] = '\0';
  		inputParms = iParms;
  		inputParms->OptionalFile = NULL;
  		inputParms->targetInfo.nTargetFiles = -1;
		inputParms->targetInfo.TargetFilenames = NULL;
  		copyInput = copyInputFiles; 
  		if (copyInput)
  			reportError(CopyAllInputMsg, moduleName, INFO);
  		//else
  		//	printf("copy files is false\n");
        yyparse();
        ConvertTargetFileNamesToList();
        return ParseError;

} 
